var _r = require('./'),
    _ = require('underscore'),
    Immutable = require('immutable'),
    Vector = Immutable.Vector;

// Register iterator dispatch of Vector to values().
//
// Returns an iterator that has next function
// and returns {value, done}.  Default looks for
// object with iterator Symbol (or '@@iterator')
_r.iterator.register(function(obj){
  if(obj instanceof Vector){
    return obj.values();
  }
});

// Register empty 
// Returns empty object of the same type as argument.
// Default returns [] if _.isArray or undefined, {} if _.isObject
// and an internal sentinel to ignore otherwise
_r.empty.register(function(obj){
  if(obj instanceof Vector){
    return Vector.empty();
  }
});

_r.append.register(function(obj, item){
  if(obj instanceof Vector){
    return obj.push(item);
  }
});

function mult(x){
  return function(y){
    return x * y;
  }
}

function isEven(x){
  return !(x % 2);
}

var vector = Vector(1,2,3,4);

// Vector [ 1, 2, 3, 4 ]
console.log(_r.sequence(vector));

// Vector [ 3, 6, 9, 12 ]
console.log(_r.sequence(_r.map(mult(3)), vector));

// Vector [ 7, 14, 21, 28 ]
console.log(_r(vector).map(mult(7)).sequence());

// Vector [ 14, 28 ]
console.log(_r(vector).map(mult(7)).filter(isEven).sequence());

// Vector [ 1, 2, 3, 4 ]
console.log(_r(vector).sequence());

// [ 1, 2, 3, 4 ]
console.log(_r(vector).value());

_r.empty.register(function(obj){
  if(_.isUndefined(obj)){
    return Vector.empty();
  }
});

// Vector [ 1, 2, 3, 4 ]
console.log(_r(vector).value());
